@@ -4,7 +4,7 @@
  * Copyright (C) 2016 Mario Luzeiro <mrluzeiro@ua.pt>
  * Copyright (C) 2015 Cirilo Bernardo <cirilo.bernardo@gmail.com>
  * Copyright (C) 2017 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2015-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,18 +36,22 @@
 #include <board.h>
 #include <common_ogl/ogl_attr_list.h>
 #include <dpi_scaling_common.h>
+#include <lset.h>
 #include <pgm_base.h>
 #include <project_pcb.h>
 #include <settings/common_settings.h>
 #include <settings/settings_manager.h>
 #include <widgets/wx_infobar.h>
+#include <widgets/std_bitmap_button.h>
 #include <eda_3d_viewer_settings.h>
 #include <board_design_settings.h>
 
+#include <3d_navlib/nl_footprint_properties_plugin.h>
+
 PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAME* aFrame,
                                                 FOOTPRINT* aFootprint,
                                                 std::vector<FP_3DMODEL>* aParentModelList ) :
-        PANEL_PREVIEW_3D_MODEL_BASE( aParent, wxID_ANY ),
+        PANEL_PREVIEW_3D_MODEL_BASE( aParent, PANEL_PREVIEW_3D_MODEL_ID ),
         m_parentFrame( aFrame ),
         m_previewPane( nullptr ),
         m_infobar( nullptr ),
@@ -86,7 +90,7 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
     m_bpvRight->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_right ) );
     m_bpvBottom->SetBitmap( KiBitmapBundle( BITMAPS::axis3d_bottom ) );
     m_bpvISO->SetBitmap( KiBitmapBundle( BITMAPS::ortho ) );
-    m_bpvBodyStyle->SetBitmap( KiBitmapBundle( BITMAPS::axis3d ) );
+    m_bpvBodyStyle->SetBitmap( KiBitmapBundle( BITMAPS::show_board_body ) );
     m_bpUpdate->SetBitmap( KiBitmapBundle( BITMAPS::reload ) );
     m_bpSettings->SetBitmap( KiBitmapBundle( BITMAPS::options_3drender ) );
 
@@ -113,7 +117,7 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
     // Ensure the footprint is shown like in Fp editor: rot 0, not flipped
     // to avoid mistakes when setting the3D shape position/rotation
     if( m_dummyFootprint->IsFlipped() )
-        m_dummyFootprint->Flip( m_dummyFootprint->GetPosition(), false );
+        m_dummyFootprint->Flip( m_dummyFootprint->GetPosition(), FLIP_DIRECTION::TOP_BOTTOM );
 
     m_dummyFootprint->SetOrientation( ANGLE_0 );
 
@@ -123,11 +127,24 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
     // Create the 3D canvas
     m_previewPane = new EDA_3D_CANVAS( this,
                                        OGL_ATT_LIST::GetAttributesList( ANTIALIASING_MODE::AA_8X ),
-                                       m_boardAdapter, m_currentCamera, PROJECT_PCB::Get3DCacheManager( &aFrame->Prj() ) );
+                                       m_boardAdapter, m_currentCamera,
+                                       PROJECT_PCB::Get3DCacheManager( &aFrame->Prj() ) );
+
+    try
+    {
+        m_spaceMouse = std::make_unique<NL_FOOTPRINT_PROPERTIES_PLUGIN>( m_previewPane );
+        m_spaceMouse->SetFocus( true );
+    }
+    catch( const std::system_error& e )
+    {
+        wxLogTrace( wxT( "KI_TRACE_NAVLIB" ), e.what() );
+    }
 
     m_boardAdapter.SetBoard( m_dummyBoard );
     m_boardAdapter.m_IsBoardView = false;
-    m_boardAdapter.m_IsPreviewer = true;   // Force display 3D models, regardless the 3D viewer options
+
+    // Force display 3D models, regardless the 3D viewer options.
+    m_boardAdapter.m_IsPreviewer = true;
 
     loadSettings();
 
@@ -162,25 +179,7 @@ PANEL_PREVIEW_3D_MODEL::PANEL_PREVIEW_3D_MODEL( wxWindow* aParent, PCB_BASE_FRAM
                      wxCommandEventHandler( PANEL_PREVIEW_3D_MODEL::onUnitsChanged ),
                      nullptr, this );
 
-#ifdef __WXOSX__
-    // Call layout once to get the proper button sizes after the bitmaps have been set
-    Layout();
-
-    // The rounded-button style used has a small border on the left/right sides.
-    // This is automatically fixed in wx for buttons with a bitmap < 20, but not
-    // when the bitmap is set to be 26x26.
-    wxSize borderFix = wxSize( 4, 4 );
-
-    m_bpvTop->SetMinSize( m_bpvTop->GetSize() + borderFix );
-    m_bpvFront->SetMinSize( m_bpvFront->GetSize() + borderFix );
-    m_bpvBack->SetMinSize( m_bpvBack->GetSize() + borderFix );
-    m_bpvLeft->SetMinSize( m_bpvLeft->GetSize() + borderFix );
-    m_bpvRight->SetMinSize( m_bpvRight->GetSize() + borderFix );
-    m_bpvBottom->SetMinSize( m_bpvBottom->GetSize() + borderFix );
-    m_bpvISO->SetMinSize( m_bpvISO->GetSize() + borderFix );
-    m_bpUpdate->SetMinSize( m_bpUpdate->GetSize() + borderFix );
-    m_bpSettings->SetMinSize( m_bpSettings->GetSize() + borderFix );
-#endif
+    Bind( wxCUSTOM_PANEL_SHOWN_EVENT, &PANEL_PREVIEW_3D_MODEL::onPanelShownEvent, this );
 }
 
 
@@ -210,13 +209,11 @@ void PANEL_PREVIEW_3D_MODEL::loadSettings()
 
     COMMON_SETTINGS* settings = Pgm().GetCommonSettings();
 
-    const DPI_SCALING_COMMON dpi{ settings, this };
-    m_previewPane->SetScaleFactor( dpi.GetScaleFactor() );
-
     // TODO(JE) use all control options
     m_boardAdapter.m_MousewheelPanning = settings->m_Input.scroll_modifier_zoom != 0;
 
-    EDA_3D_VIEWER_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<EDA_3D_VIEWER_SETTINGS>();
+    SETTINGS_MANAGER&       mgr = Pgm().GetSettingsManager();
+    EDA_3D_VIEWER_SETTINGS* cfg = mgr.GetAppSettings<EDA_3D_VIEWER_SETTINGS>( "3d_viewer" );
 
     if( cfg )
     {
@@ -237,6 +234,7 @@ void PANEL_PREVIEW_3D_MODEL::loadSettings()
         cfg->m_Render.show_solderpaste = m_bodyStyleShowAll;
         cfg->m_Render.show_zones = m_bodyStyleShowAll;
         cfg->m_Render.show_board_body = m_bodyStyleShowAll;
+        cfg->m_Render.use_board_editor_copper_colors = false;
     }
 }
 
@@ -248,7 +246,8 @@ void PANEL_PREVIEW_3D_MODEL::loadSettings()
  */
 static double rotationFromString( const wxString& aValue )
 {
-    double rotation = EDA_UNIT_UTILS::UI::DoubleValueFromString( unityScale, EDA_UNITS::DEGREES, aValue );
+    double rotation = EDA_UNIT_UTILS::UI::DoubleValueFromString( unityScale, EDA_UNITS::DEGREES,
+                                                                 aValue );
 
     if( rotation > MAX_ROTATION )
     {
@@ -274,6 +273,10 @@ wxString PANEL_PREVIEW_3D_MODEL::formatScaleValue( double aValue )
 
 wxString PANEL_PREVIEW_3D_MODEL::formatRotationValue( double aValue )
 {
+    // Sigh.  Did we really need differentiated +/- 0.0?
+    if( aValue == -0.0 )
+        aValue = 0.0;
+
     return wxString::Format( wxT( "%.2f%s" ),
                              aValue,
                              EDA_UNIT_UTILS::GetText( EDA_UNITS::DEGREES ) );
@@ -283,7 +286,7 @@ wxString PANEL_PREVIEW_3D_MODEL::formatRotationValue( double aValue )
 wxString PANEL_PREVIEW_3D_MODEL::formatOffsetValue( double aValue )
 {
     // Convert from internal units (mm) to user units
-    if( m_userUnits == EDA_UNITS::INCHES )
+    if( m_userUnits == EDA_UNITS::INCH )
         aValue /= 25.4;
     else if( m_userUnits == EDA_UNITS::MILS )
         aValue /= 25.4 / 1e3;
@@ -307,9 +310,11 @@ void PANEL_PREVIEW_3D_MODEL::SetSelectedModel( int idx )
         yscale->ChangeValue( formatScaleValue( modelInfo.m_Scale.y ) );
         zscale->ChangeValue( formatScaleValue( modelInfo.m_Scale.z ) );
 
-        xrot->ChangeValue( formatRotationValue( modelInfo.m_Rotation.x ) );
-        yrot->ChangeValue( formatRotationValue( modelInfo.m_Rotation.y ) );
-        zrot->ChangeValue( formatRotationValue( modelInfo.m_Rotation.z ) );
+        // Rotation is stored in the file as positive-is-CW, but we use positive-is-CCW in the GUI
+        // to match the rest of KiCad
+        xrot->ChangeValue( formatRotationValue( -modelInfo.m_Rotation.x ) );
+        yrot->ChangeValue( formatRotationValue( -modelInfo.m_Rotation.y ) );
+        zrot->ChangeValue( formatRotationValue( -modelInfo.m_Rotation.z ) );
 
         xoff->ChangeValue( formatOffsetValue( modelInfo.m_Offset.x ) );
         yoff->ChangeValue( formatOffsetValue( modelInfo.m_Offset.y ) );
@@ -352,9 +357,11 @@ void PANEL_PREVIEW_3D_MODEL::updateOrientation( wxCommandEvent &event )
         modelInfo->m_Scale.z = EDA_UNIT_UTILS::UI::DoubleValueFromString(
                 pcbIUScale, EDA_UNITS::UNSCALED, zscale->GetValue() );
 
-        modelInfo->m_Rotation.x = rotationFromString( xrot->GetValue() );
-        modelInfo->m_Rotation.y = rotationFromString( yrot->GetValue() );
-        modelInfo->m_Rotation.z = rotationFromString( zrot->GetValue() );
+        // Rotation is stored in the file as positive-is-CW, but we use positive-is-CCW in the GUI
+        // to match the rest of KiCad
+        modelInfo->m_Rotation.x = -rotationFromString( xrot->GetValue() );
+        modelInfo->m_Rotation.y = -rotationFromString( yrot->GetValue() );
+        modelInfo->m_Rotation.z = -rotationFromString( zrot->GetValue() );
 
         modelInfo->m_Offset.x = EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, m_userUnits,
                                                                            xoff->GetValue() )
@@ -368,6 +375,7 @@ void PANEL_PREVIEW_3D_MODEL::updateOrientation( wxCommandEvent &event )
 
         // Update the dummy footprint for the preview
         UpdateDummyFootprint( false );
+        onModify();
     }
 }
 
@@ -383,13 +391,14 @@ void PANEL_PREVIEW_3D_MODEL::onOpacitySlider( wxCommandEvent& event )
 
         // Update the dummy footprint for the preview
         UpdateDummyFootprint( false );
+        onModify();
     }
 }
 
 
 void PANEL_PREVIEW_3D_MODEL::setBodyStyleView( wxCommandEvent& event )
 {
-    // turn ON or OFF options to show the board body if OFF, soder paste, soldermask
+    // turn ON or OFF options to show the board body if OFF, solder paste, soldermask
     // and board body are hidden, to allows a good view of the 3D model and its pads.
     EDA_3D_VIEWER_SETTINGS* cfg = m_boardAdapter.m_Cfg;
 
@@ -444,10 +453,15 @@ void PANEL_PREVIEW_3D_MODEL::doIncrementScale( wxSpinEvent& event, double aSign
     else if( spinCtrl == m_spinZscale )
         textCtrl = zscale;
 
+    double step = SCALE_INCREMENT;
+
+    if( wxGetMouseState().ShiftDown( ) )
+        step = SCALE_INCREMENT_FINE;
+
     double curr_value = EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, EDA_UNITS::UNSCALED,
                                                                    textCtrl->GetValue() );
 
-    curr_value += ( SCALE_INCREMENT * aSign );
+    curr_value += ( step * aSign );
     curr_value = std::max( 1/MAX_SCALE, curr_value );
     curr_value = std::min( curr_value, MAX_SCALE );
 
@@ -465,10 +479,15 @@ void PANEL_PREVIEW_3D_MODEL::doIncrementRotation( wxSpinEvent& aEvent, double aS
     else if( spinCtrl == m_spinZrot )
         textCtrl = zrot;
 
+    double step = ROTATION_INCREMENT;
+
+    if( wxGetMouseState().ShiftDown( ) )
+        step = ROTATION_INCREMENT_FINE;
+
     double curr_value = EDA_UNIT_UTILS::UI::DoubleValueFromString( unityScale, EDA_UNITS::DEGREES,
                                                                    textCtrl->GetValue() );
 
-    curr_value += ( ROTATION_INCREMENT * aSign );
+    curr_value += ( step * aSign );
     curr_value = std::max( -MAX_ROTATION, curr_value );
     curr_value = std::min( curr_value, MAX_ROTATION );
 
@@ -488,16 +507,22 @@ void PANEL_PREVIEW_3D_MODEL::doIncrementOffset( wxSpinEvent& event, double aSign
         textCtrl = zoff;
 
     double step_mm = OFFSET_INCREMENT_MM;
-    double curr_value_mm =
-            EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, m_userUnits,
-                                                       textCtrl->GetValue() )
-            / pcbIUScale.IU_PER_MM;
 
-    if( m_userUnits == EDA_UNITS::MILS || m_userUnits == EDA_UNITS::INCHES )
+    if( wxGetMouseState().ShiftDown( ) )
+        step_mm = OFFSET_INCREMENT_MM_FINE;
+
+    if( m_userUnits == EDA_UNITS::MILS || m_userUnits == EDA_UNITS::INCH )
     {
         step_mm = 25.4*OFFSET_INCREMENT_MIL/1000;
+
+        if( wxGetMouseState().ShiftDown( ) )
+            step_mm = 25.4*OFFSET_INCREMENT_MIL_FINE/1000;;
     }
 
+    double curr_value_mm = EDA_UNIT_UTILS::UI::DoubleValueFromString( pcbIUScale, m_userUnits,
+                                                                      textCtrl->GetValue() )
+                           / pcbIUScale.IU_PER_MM;
+
     curr_value_mm += ( step_mm * aSign );
     curr_value_mm = std::max( -MAX_OFFSET, curr_value_mm );
     curr_value_mm = std::min( curr_value_mm, MAX_OFFSET );
@@ -533,10 +558,10 @@ void PANEL_PREVIEW_3D_MODEL::onMouseWheelRot( wxMouseEvent& event )
 {
     wxTextCtrl* textCtrl = (wxTextCtrl*) event.GetEventObject();
 
-    double step = ROTATION_INCREMENT_WHEEL;
+    double step = ROTATION_INCREMENT;
 
     if( event.ShiftDown( ) )
-        step = ROTATION_INCREMENT_WHEEL_FINE;
+        step = ROTATION_INCREMENT_FINE;
 
     if( event.GetWheelRotation() >= 0 )
         step = -step;
@@ -561,7 +586,7 @@ void PANEL_PREVIEW_3D_MODEL::onMouseWheelOffset( wxMouseEvent& event )
     if( event.ShiftDown( ) )
         step_mm = OFFSET_INCREMENT_MM_FINE;
 
-    if( m_userUnits == EDA_UNITS::MILS || m_userUnits == EDA_UNITS::INCHES )
+    if( m_userUnits == EDA_UNITS::MILS || m_userUnits == EDA_UNITS::INCH )
     {
         step_mm = 25.4*OFFSET_INCREMENT_MIL/1000.0;
 
@@ -607,6 +632,17 @@ void PANEL_PREVIEW_3D_MODEL::onUnitsChanged( wxCommandEvent& aEvent )
 }
 
 
+void PANEL_PREVIEW_3D_MODEL::onPanelShownEvent( wxCommandEvent& aEvent )
+{
+    if( m_spaceMouse )
+    {
+        m_spaceMouse->SetFocus( static_cast<bool>( aEvent.GetInt() ) );
+    }
+
+    aEvent.Skip();
+}
+
+
 void PANEL_PREVIEW_3D_MODEL::UpdateDummyFootprint( bool aReloadRequired )
 {
     m_dummyFootprint->Models().clear();
@@ -622,3 +658,12 @@ void PANEL_PREVIEW_3D_MODEL::UpdateDummyFootprint( bool aReloadRequired )
 
     m_previewPane->Request_refresh();
 }
+
+
+void PANEL_PREVIEW_3D_MODEL::onModify()
+{
+    KIWAY_HOLDER* kiwayHolder = dynamic_cast<KIWAY_HOLDER*>( wxGetTopLevelParent( this ) );
+
+    if( kiwayHolder && kiwayHolder->GetType() == KIWAY_HOLDER::DIALOG )
+        static_cast<DIALOG_SHIM*>( kiwayHolder )->OnModify();
+}
