@@ -23,7 +23,8 @@
 #include <common.h>
 #include <widgets/wx_grid.h>
 #include <sch_reference_list.h>
-#include <schematic_settings.h>
+#include <sch_commit.h>
+#include <sch_screen.h>
 #include "string_utils.h"
 
 #include "fields_data_model.h"
@@ -60,9 +61,13 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::updateDataStoreSymbolField( const SCH_SYMBOL
     else if( const SCH_FIELD* field = aSymbol.GetFieldByName( aFieldName ) )
     {
         if( field->IsPrivate() )
+        {
             m_dataStore[aSymbol.m_Uuid][aFieldName] = wxEmptyString;
-        else
-            m_dataStore[aSymbol.m_Uuid][aFieldName] = field->GetText();
+            return;
+        }
+
+        wxString value = aSymbol.Schematic()->ConvertKIIDsToRefs( field->GetText() );
+        m_dataStore[aSymbol.m_Uuid][aFieldName] = value;
     }
     else if( IsTextVar( aFieldName ) )
     {
@@ -108,19 +113,19 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::RenameColumn( int aCol, const wxString& newN
 }
 
 
-int FIELDS_EDITOR_GRID_DATA_MODEL::GetFieldNameCol( const wxString& aFieldName )
+int FIELDS_EDITOR_GRID_DATA_MODEL::GetFieldNameCol( const wxString& aFieldName ) const
 {
     for( size_t i = 0; i < m_cols.size(); i++ )
     {
         if( m_cols[i].m_fieldName == aFieldName )
-            return (int) i;
+            return static_cast<int>( i );
     }
 
     return -1;
 }
 
 
-const std::vector<BOM_FIELD> FIELDS_EDITOR_GRID_DATA_MODEL::GetFieldsOrdered()
+std::vector<BOM_FIELD> FIELDS_EDITOR_GRID_DATA_MODEL::GetFieldsOrdered()
 {
     std::vector<BOM_FIELD> fields;
 
@@ -135,6 +140,8 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::SetFieldsOrder( const std::vector<wxString>&
 {
     size_t foundCount = 0;
 
+    wxCHECK( aNewOrder.size() == m_cols.size(), /* void */ );
+
     for( const wxString& newField : aNewOrder )
     {
         for( size_t i = 0; i < m_cols.size(); i++ )
@@ -170,6 +177,29 @@ wxString FIELDS_EDITOR_GRID_DATA_MODEL::GetValue( int aRow, int aCol )
 }
 
 
+wxGridCellAttr* FIELDS_EDITOR_GRID_DATA_MODEL::GetAttr( int aRow, int aCol,
+                                                        wxGridCellAttr::wxAttrKind aKind )
+{
+    if( GetColFieldName( aCol ) == GetCanonicalFieldName( DATASHEET_FIELD )
+            || IsURL( GetValue( m_rows[aRow], aCol ) ) )
+    {
+        if( m_urlEditor )
+        {
+            m_urlEditor->IncRef();
+            return enhanceAttr( m_urlEditor, aRow, aCol, aKind );
+        }
+    }
+
+    if( m_colAttrs[aCol] )
+    {
+        m_colAttrs[aCol]->IncRef();
+        return enhanceAttr( m_colAttrs[aCol], aRow, aCol, aKind );
+    }
+
+    return nullptr;
+}
+
+
 wxString FIELDS_EDITOR_GRID_DATA_MODEL::GetValue( const DATA_MODEL_ROW& group, int aCol,
                                                   const wxString& refDelimiter,
                                                   const wxString& refRangeDelimiter,
@@ -275,7 +305,7 @@ wxString FIELDS_EDITOR_GRID_DATA_MODEL::GetValue( const DATA_MODEL_ROW& group, i
 
 void FIELDS_EDITOR_GRID_DATA_MODEL::SetValue( int aRow, int aCol, const wxString& aValue )
 {
-    wxCHECK_RET( aCol >= 0 && aCol < (int) m_cols.size(), wxS( "Invalid column number" ) );
+    wxCHECK_RET( aCol >= 0 && aCol < static_cast<int>( m_cols.size() ), wxS( "Invalid column number" ) );
 
     // Can't modify references or text variables column, e.g. ${QUANTITY}
     if( ColIsReference( aCol )
@@ -295,35 +325,35 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::SetValue( int aRow, int aCol, const wxString
 
 bool FIELDS_EDITOR_GRID_DATA_MODEL::ColIsReference( int aCol )
 {
-    wxCHECK( aCol >= 0 && aCol < (int) m_cols.size(), false );
+    wxCHECK( aCol >= 0 && aCol < static_cast<int>( m_cols.size() ), false );
     return m_cols[aCol].m_fieldName == GetCanonicalFieldName( REFERENCE_FIELD );
 }
 
 
 bool FIELDS_EDITOR_GRID_DATA_MODEL::ColIsValue( int aCol )
 {
-    wxCHECK( aCol >= 0 && aCol < (int) m_cols.size(), false );
+    wxCHECK( aCol >= 0 && aCol < static_cast<int>( m_cols.size() ), false );
     return m_cols[aCol].m_fieldName == GetCanonicalFieldName( VALUE_FIELD );
 }
 
 
 bool FIELDS_EDITOR_GRID_DATA_MODEL::ColIsQuantity( int aCol )
 {
-    wxCHECK( aCol >= 0 && aCol < (int) m_cols.size(), false );
+    wxCHECK( aCol >= 0 && aCol < static_cast<int>( m_cols.size() ), false );
     return m_cols[aCol].m_fieldName == QUANTITY_VARIABLE;
 }
 
 
 bool FIELDS_EDITOR_GRID_DATA_MODEL::ColIsItemNumber( int aCol )
 {
-    wxCHECK( aCol >= 0 && aCol < (int) m_cols.size(), false );
+    wxCHECK( aCol >= 0 && aCol < static_cast<int>( m_cols.size() ), false );
     return m_cols[aCol].m_fieldName == ITEM_NUMBER_VARIABLE;
 }
 
 
 bool FIELDS_EDITOR_GRID_DATA_MODEL::ColIsAttribute( int aCol )
 {
-    wxCHECK( aCol >= 0 && aCol < (int) m_cols.size(), false );
+    wxCHECK( aCol >= 0 && aCol < static_cast<int>( m_cols.size() ), false );
     return isAttribute( m_cols[aCol].m_fieldName );
 }
 
@@ -441,7 +471,7 @@ bool FIELDS_EDITOR_GRID_DATA_MODEL::groupMatch( const SCH_REFERENCE& lhRef,
     for( size_t i = 0; i < m_cols.size(); ++i )
     {
         //Handled already
-        if( (int) i == refCol )
+        if( static_cast<int>( i ) == refCol )
             continue;
 
         if( !m_cols[i].m_group )
@@ -757,22 +787,18 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::ExpandAfterSort()
 }
 
 
-void FIELDS_EDITOR_GRID_DATA_MODEL::ApplyData(
-        std::function<void( SCH_SYMBOL&, SCH_SHEET_PATH& )> symbolChangeHandler )
+void FIELDS_EDITOR_GRID_DATA_MODEL::ApplyData( SCH_COMMIT& aCommit )
 {
     for( unsigned i = 0; i < m_symbolsList.GetCount(); ++i )
     {
         SCH_SYMBOL& symbol = *m_symbolsList[i].GetSymbol();
 
-        symbolChangeHandler( symbol, m_symbolsList[i].GetSheetPath() );
+        aCommit.Modify( &symbol, m_symbolsList[i].GetSheetPath().LastScreen() );
 
         const std::map<wxString, wxString>& fieldStore = m_dataStore[symbol.m_Uuid];
 
-        for( const std::pair<wxString, wxString> srcData : fieldStore )
+        for( const auto& [srcName, srcValue] : fieldStore )
         {
-            const wxString& srcName = srcData.first;
-            const wxString& srcValue = srcData.second;
-
             // Attributes bypass the field logic, so handle them first
             if( isAttribute( srcName ) )
             {
@@ -813,24 +839,13 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::ApplyData(
             if( destField->GetId() == REFERENCE_FIELD )
             {
                 // Reference is not editable from this dialog
+                continue;
             }
-            else if( destField->GetId() == VALUE_FIELD )
-            {
-                // Value field cannot be empty
-                if( !srcValue.IsEmpty() )
-                    symbol.SetValueFieldText( srcValue );
-            }
-            else if( destField->GetId() == FOOTPRINT_FIELD )
-            {
-                symbol.SetFootprintFieldText( srcValue );
-            }
-            else
-            {
-                destField->SetText( srcValue );
-            }
+
+            destField->SetText( symbol.Schematic()->ConvertRefsToKIIDs( srcValue ) );
         }
 
-        for( int ii = symbol.GetFields().size() - 1; ii >= 0; ii-- )
+        for( int ii = static_cast<int>( symbol.GetFields().size() ) - 1; ii >= 0; ii-- )
         {
             if( symbol.GetFields()[ii].IsMandatory() || symbol.GetFields()[ii].IsPrivate() )
                 continue;
@@ -879,15 +894,17 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::ApplyBomPreset( const BOM_PRESET& aPreset )
         SetGroupColumn( i, false );
     }
 
+    std::set<wxString> seen;
     std::vector<wxString> order;
 
     // Set columns that are present and shown
-    for( BOM_FIELD field : aPreset.fieldsOrdered )
+    for( const BOM_FIELD& field : aPreset.fieldsOrdered )
     {
         // Ignore empty fields
-        if(!field.name)
+        if( !field.name || seen.count( field.name ) )
             continue;
 
+        seen.insert( field.name );
         order.emplace_back( field.name );
 
         int col = GetFieldNameCol( field.name );
@@ -900,7 +917,9 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::ApplyBomPreset( const BOM_PRESET& aPreset )
             col = GetFieldNameCol( field.name );
         }
         else
+        {
             SetColLabelValue( col, field.label );
+        }
 
         SetGroupColumn( col, field.groupBy );
         SetShowColumn( col, field.show );
@@ -956,7 +975,7 @@ wxString FIELDS_EDITOR_GRID_DATA_MODEL::Export( const BOM_FMT_PRESET& settings )
     for( size_t col = 0; col < m_cols.size(); col++ )
     {
         if( m_cols[col].m_show )
-            last_col = (int) col;
+            last_col = static_cast<int>( col );
     }
 
     // No shown columns
@@ -978,8 +997,10 @@ wxString FIELDS_EDITOR_GRID_DATA_MODEL::Export( const BOM_FMT_PRESET& settings )
                 }
 
                 if( !settings.stringDelimiter.IsEmpty() )
+                {
                     field.Replace( settings.stringDelimiter,
                                    settings.stringDelimiter + settings.stringDelimiter );
+                }
 
                 return settings.stringDelimiter + field + settings.stringDelimiter
                        + ( last ? wxString( wxS( "\n" ) ) : settings.fieldDelimiter );
@@ -991,14 +1012,14 @@ wxString FIELDS_EDITOR_GRID_DATA_MODEL::Export( const BOM_FMT_PRESET& settings )
         if( !m_cols[col].m_show )
             continue;
 
-        out.Append( formatField( m_cols[col].m_label, col == (size_t) last_col ) );
+        out.Append( formatField( m_cols[col].m_label, col == static_cast<size_t>( last_col ) ) );
     }
 
     // Data rows
     for( size_t row = 0; row < m_rows.size(); row++ )
     {
         // Don't output child rows
-        if( GetRowFlags( (int) row ) == CHILD_ITEM )
+        if( GetRowFlags( static_cast<int>( row ) ) == CHILD_ITEM )
             continue;
 
         for( size_t col = 0; col < m_cols.size(); col++ )
@@ -1006,10 +1027,10 @@ wxString FIELDS_EDITOR_GRID_DATA_MODEL::Export( const BOM_FMT_PRESET& settings )
             if( !m_cols[col].m_show )
                 continue;
 
-            // Get the unanottated version of the field, e.g. no ">   " or "v   " by
-            out.Append( formatField( GetExportValue( (int) row, (int) col, settings.refDelimiter,
-                                                     settings.refRangeDelimiter ),
-                                     col == (size_t) last_col ) );
+            // Get the unannotated version of the field, e.g. no ">   " or "v   " by
+            out.Append( formatField( GetExportValue( static_cast<int>( row ), static_cast<int>( col ),
+                                                     settings.refDelimiter, settings.refRangeDelimiter ),
+                                     col == static_cast<size_t>( last_col ) ) );
         }
     }
 
@@ -1023,13 +1044,20 @@ void FIELDS_EDITOR_GRID_DATA_MODEL::AddReferences( const SCH_REFERENCE_LIST& aRe
     {
         if( !m_symbolsList.Contains( ref ) )
         {
+            SCH_SYMBOL* symbol = ref.GetSymbol();
+
             m_symbolsList.AddItem( ref );
 
             // Update the fields of every reference
-            for( const SCH_FIELD& field : ref.GetSymbol()->GetFields() )
+            for( const SCH_FIELD& field : symbol->GetFields() )
             {
                 if( !field.IsPrivate() )
-                    m_dataStore[ref.GetSymbol()->m_Uuid][field.GetCanonicalName()] = field.GetText();
+                {
+                    wxString name = field.GetCanonicalName();
+                    wxString value = symbol->Schematic()->ConvertKIIDsToRefs( field.GetText() );
+
+                    m_dataStore[symbol->m_Uuid][name] = value;
+                }
             }
         }
     }
