@@ -81,7 +81,7 @@ BOARD::BOARD() :
         m_timeStamp( 1 ),
         m_paper( PAGE_INFO::A4 ),
         m_project( nullptr ),
-        m_userUnits( EDA_UNITS::MILLIMETRES ),
+        m_userUnits( EDA_UNITS::MM ),
         m_designSettings( new BOARD_DESIGN_SETTINGS( nullptr, "board.design_settings" ) ),
         m_NetInfo( this ),
         m_embedFonts( false )
@@ -538,18 +538,15 @@ void BOARD::RunOnDescendants( const std::function<void ( BOARD_ITEM* )>& aFuncti
         for( PCB_MARKER* marker : m_markers )
             aFunction( marker );
 
+        for( PCB_GROUP* group : m_groups )
+            aFunction( group );
+
         for( FOOTPRINT* footprint : m_footprints )
         {
             aFunction( footprint );
             footprint->RunOnDescendants( aFunction, aDepth + 1 );
         }
 
-        for( PCB_GROUP* group : m_groups )
-        {
-            aFunction( group );
-            group->RunOnDescendants( aFunction, aDepth + 1 );
-        }
-
         for( BOARD_ITEM* drawing : m_drawings )
         {
             aFunction( drawing );
@@ -1187,6 +1184,23 @@ void BOARD::FinalizeBulkRemove( std::vector<BOARD_ITEM*>& aRemovedItems )
 }
 
 
+void BOARD::BulkRemoveStaleTeardrops( BOARD_COMMIT& aCommit )
+{
+    for( int ii = (int) m_zones.size() - 1; ii >= 0; --ii )
+    {
+        ZONE* zone = m_zones[ii];
+
+        if( zone->IsTeardropArea() && zone->HasFlag( STRUCT_DELETED ) )
+        {
+            m_itemByIdCache.erase( zone->m_Uuid );
+            m_zones.erase( m_zones.begin() + ii );
+            m_connectivity->Remove( zone );
+            aCommit.Removed( zone );
+        }
+    }
+}
+
+
 void BOARD::Remove( BOARD_ITEM* aBoardItem, REMOVE_MODE aRemoveMode )
 {
     // find these calls and fix them!  Don't send me no stinking' nullptr.
