@@ -50,10 +50,10 @@ enum class ALTIUM_UNIT
 {
     UNKNOWN = 0,
 
-    INCHES      = 1,
-    MILS        = 2,
-    MILLIMETERS = 3,
-    CENTIMETER  = 4
+    INCH    = 1,
+    MILS    = 2,
+    MM      = 3,
+    CM      = 4
 };
 
 enum class ALTIUM_CLASS_KIND
