@@ -223,7 +223,9 @@ static bool testFootprintSelector( FOOTPRINT* aFp, const wxString& aSelector )
     {
         wxString name = aSelector.Mid( 8, aSelector.Length() - 9 );
 
-        if( aFp->GetComponentClass()->ContainsClassName( name ) )
+        const COMPONENT_CLASS* compClass = aFp->GetComponentClass();
+
+        if( compClass && compClass->ContainsClassName( name ) )
             return true;
     }
     else if( aFp->GetReference().Matches( aSelector ) )
@@ -1261,7 +1263,12 @@ static void hasComponentClassFunc( LIBEVAL::CONTEXT* aCtx, void* self )
     result->SetDeferredEval(
             [item, arg]() -> double
             {
-                FOOTPRINT* footprint = dynamic_cast<FOOTPRINT*>( item );
+                FOOTPRINT* footprint = nullptr;
+
+                if( item->Type() == PCB_FOOTPRINT_T )
+                    footprint = static_cast<FOOTPRINT*>( item );
+                else
+                    footprint = item->GetParentFootprint();
 
                 if( !footprint )
                     return 0.0;
